// Persistence of Vision Ray Tracer Scene Description File
// File: bspline_ex.pov
// Vers: 3.6.1
// Desc: Example describing my BSpline-Macros
// Date: 28.03.02 (dd.mm.yy)
// Auth: Tim Nikias Wenclawiak - www.NoLights.de
// Last Update: 02.12.04 (dd.mm.yy)

//Required files:
// "bsplines.inc" - Include-File for the BSplines

/*Description:
Setup for BSplines
Its actually rather simple. In an array with a minimum of 4 vectors,
you create pairs: a node and a corresponding direction for the spline.
Note that you actually give a direction, no control-node in 3D-Space!

For further enhancements, you may declare the nodes and direction
with a 4-component-vector (<x,y,z,t>). When calculating the
curves, the fourth component of the directions will be used to
modify the tangent of the curve, but only when the current node is
the beginning of a section, e.g. the tangent given by xyz will be
used if it's the end, xyz*(t+1) is the tangent if it's the start.

To grasp the concept, here's the way the bezier-splines work:
every four vectors (2xNodes + 2xDirections) generate one curved
path. The Macros implement an easy way of adding just two new
vectors (1*Node + 1*Direction), and it will use the last 2 vectors
as the start for the next curve if another pair of vectors follow.

In order to have the curves still join smoothly, they need to use
the same tangent, but its strength may be varied. The fourth-component
(t) will be used for that. A value of 0 will change nothing, a value
of 1 increase the strength by 100% percent, a value of -.5 decrease
by 50% percent.

Example:
1.    Node   + Direction
  < 2,0,0,0> , <0, 1,0,0>,
2.    Node   + Direction
  <-2,0,0,0> , <0,-1,0,1>,
3.    Node   + Direction
  < 2,0,0,0> , <0, 1,0,0>

These six vectors will result in two curves:
The first begins at x*2 with a tangent of y,
 and ends at x*-2 with a tangent of -y.
The second begins at x*-2 with a tangent of -y*2
 (deriven from -y*(t+1)=-y*(2)=-y*2) and ends
 at x*2 with a tangent of y.
*/

#include "bsplines.inc"

global_settings{assumed_gamma 1}
camera{location <0,0,-10> look_at <0,0,0>}
light_source{<10,20,-50> rgb 1}

#declare BSpline_Array=array[6]
{
 <-2,0,0,0>,<-2, 3,0,0>, //Node + Direction
 < 0,0,0,0>,<-3,-4,0,-.5>, // -> Lower the strength of the tangent at this node by 50%
 < 2,0,0,0>,< 1, 1,0,0>
}

#declare Comp_BSpline_Array=array[6]
{
 <-2,0,0,0>,<-2, 3,0,0>, //Node + Direction
 < 0,0,0,0>,<-3,-4,0,0>, // -> change nothing on the tangent for a comparison
 < 2,0,0,0>,< 1, 1,0,0>
}

//Modifying macros (not changing anything in this example, just there
//to show how they are used)
BSpline_Scale4(BSpline_Array,<1,1,1>)
BSpline_Rotate4(BSpline_Array,<0,0,0>)
BSpline_Translate(BSpline_Array,<0,0,0>)

//Reverses Spline (beginning and end are switched)
// (commented out to make understanding easier)
//BSpline_Flip(BSpline_Array)

//CSG-Macros
 //Red Spheres at nodes:
 union{ Show_BNodes(BSpline_Array,.2) pigment{rgb x}}
 //green cones indicating direction at actual nodes
 union{ Show_BDirs(BSpline_Array,.15) pigment{rgb y}}
 //Blue BSpline made of 500 Spheres and Cylinders (each)
 union{ Show_BSpline(BSpline_Array,.1,500) pigment{rgb z}}
 //Spline for comparison
 union{ Show_BSpline(Comp_BSpline_Array,.09,500) pigment{rgb x}}

//Returning Macros
 //White sphere at .25 of spline
 sphere{ BSpline_Pos(BSpline_Array,.25),.3 pigment{rgb 1}}
 //White cone indicating Curvature-normal at .75 of spline
 cone{
  //Base position on spline
  BSpline_Pos(BSpline_Array,.75),.3
  //Base position on spline+normal
  BSpline_Pos(BSpline_Array,.75)+BSpline_Normal(BSpline_Array,.75),0
  pigment{rgb 1}
  }

/*File-Output
 When using the lower three lines of code, a file named "splin_ex.spl" is written.
 It has two arrays:
 BSpline_Node: original BSpline-Array
 Linear_Node: New array with series of positions on spline
 
 The example shows that the movers of linear-interpolation and bsplines
 don't match. Cubic interpolation (as done for the bsplines) has
 side-effects, like changes in "speed" when travelling with a constantly
 increasing value for the mover across the spline. The linear-interpolation
 overcomes this.
*/

//Commented out: output-macros
//BSpline2Linear(BSpline_Array,1000,20,"splin_ex.spl")
//#include "splin_ex.spl"
//sphere{ Linear_Pos(Linear_Node,.25),.25 pigment{rgb <1,1,0>}}

